"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const bf_types_1 = require("bf-types");
const decimal_js_1 = require("decimal.js");
const voca_1 = require("voca");
/* ~~~ General String Functions ~~~ */
function formatFloat(value, decimalPlaces) {
    let numberValue = NaN;
    if (!decimalPlaces) {
        numberValue = Number(value);
        if (Number.isNaN(numberValue)) {
            numberValue = 0;
        }
        return String(numberValue);
    }
    numberValue = new decimal_js_1.Decimal(Number(value)).toDP(decimalPlaces).toNumber();
    if (Number.isNaN(numberValue)) {
        numberValue = 0;
    }
    return numberValue.toFixed(decimalPlaces);
}
exports.formatFloat = formatFloat;
function numberWithCommas(str) {
    return str.replace(/\B(?=(\d{3})+(?!\d))/g, ',');
}
exports.numberWithCommas = numberWithCommas;
function formatDecimal(value, decimalPlaces = 2) {
    const rounded = formatFloat(value, decimalPlaces);
    const chunks = rounded.split('.');
    if (chunks.length < 2) {
        return numberWithCommas(rounded);
    }
    return `${numberWithCommas(chunks[0])}.${chunks[1]}`;
}
exports.formatDecimal = formatDecimal;
function toDisplay(str) {
    const split = str.replace(/_/g, ' ').toLocaleLowerCase().split(' ');
    const toUpper = split.map((s) => toUpperFirstLetter(s));
    return toUpper.join(' ');
}
exports.toDisplay = toDisplay;
function toLowerCamel(str) {
    return voca_1.camelCase(str);
}
exports.toLowerCamel = toLowerCamel;
function toUpperCamel(str) {
    return voca_1.chain(str).camelCase().capitalize().value();
}
exports.toUpperCamel = toUpperCamel;
function toLowerFirstLetter(str) {
    return voca_1.decapitalize(str);
}
exports.toLowerFirstLetter = toLowerFirstLetter;
function toUpperFirstLetter(str) {
    return voca_1.capitalize(str);
}
exports.toUpperFirstLetter = toUpperFirstLetter;
function toLowerSnakeCase(str) {
    return voca_1.snakeCase(str);
}
exports.toLowerSnakeCase = toLowerSnakeCase;
function toUpperSnakeCase(str) {
    return voca_1.chain(str).snakeCase().upperCase().value();
}
exports.toUpperSnakeCase = toUpperSnakeCase;
/* ~~~ String Functions with Application Logic ~~~ */
const domainUriMap = new Map();
const moduleUriMap = new Map();
for (const domain of Object.values(bf_types_1.DOMAINS)) {
    domainUriMap.set(domain, toLowerCamel(domain));
}
for (const module of Object.values(bf_types_1.ALL_MODULES)) {
    moduleUriMap.set(module, toLowerCamel(module));
}
function domainToUri(domain) {
    const uri = domainUriMap.get(domain);
    if (!uri) {
        throw new Error(`There is no URI for the ${domain} domain.`);
    }
    return uri;
}
exports.domainToUri = domainToUri;
function moduleToUri(module) {
    const uri = moduleUriMap.get(module);
    if (!uri) {
        throw new Error(`There is no URI for the ${module} module.`);
    }
    return uri;
}
exports.moduleToUri = moduleToUri;
function validateDomainAndModule(domain, module) {
    if (!bf_types_1.DOMAINS[domain]) {
        throw new Error(`${domain} is not a valid Entity domain`);
    }
    if (!bf_types_1.ALL_MODULES[module]) {
        throw new Error(`${module} is not a valid Entity module`);
    }
    const domainModules = bf_types_1.DOMAIN_MODULES[domain];
    if (typeof domainModules[module] !== 'string') {
        throw new Error(`${module} is not a module of the ${toDisplay(domain)}`);
    }
}
exports.validateDomainAndModule = validateDomainAndModule;
